use crate::MainWindow;
use super::{GUIElements, GUIElementsRef, MainWindowRef, display_error};
use slint::Image;
use tool_helper::Error;

pub struct FileTab {
    main_window: MainWindowRef
}

impl FileTab {
    pub fn new(main_window: MainWindowRef) -> FileTab {
        FileTab{main_window}
    }

    pub fn clear_load(&self) {
        let main_window = self.main_window.borrow();

        main_window.set_file_tab_browse_path("".into());
        main_window.set_file_tab_image_data(Image::default());
        main_window.set_file_tab_image_name("".into());
        main_window.set_file_tab_enable(false);
    }

    pub fn update_new_load(&self, file_name: Option<String>, image: Image) {
        let main_window = self.main_window.borrow();

        main_window.set_file_tab_image_data(image);
        if let Some(file_name) = file_name {
            main_window.set_file_tab_image_name(file_name.into());
        }
        
        else {
            main_window.set_file_tab_image_name("".into());
        }
        main_window.set_file_tab_enable(true);
    }

    pub fn get_file_name(&self) -> String {
        self.main_window.borrow().get_file_tab_image_name().to_string()
    }

    pub fn on_browse_file(&self, gui_elements: GUIElementsRef, mut function: impl FnMut(&mut GUIElements, &MainWindow) -> Result<(), Error> + 'static) {
        let main_window_cloned = self.main_window.clone();
        let gui_cloned         = gui_elements.clone();

        self.main_window.borrow().on_file_tab_browse_convert_image(move || {
            if let Err(error) = function(&mut gui_cloned.borrow_mut(), &main_window_cloned.borrow()) {
                display_error("Loadind file failed", &error.to_string());
            }
        });
    }

    pub fn on_add_image(&self, gui_elements: GUIElementsRef, mut function: impl FnMut(&mut GUIElements, &MainWindow) -> Result<(), Error> + 'static) {
        let main_window_cloned = self.main_window.clone();
        let gui_cloned         = gui_elements.clone();

        self.main_window.borrow().on_file_tab_add_convert_image(move || {
            if let Err(error) = function(&mut gui_cloned.borrow_mut(), &main_window_cloned.borrow()) {
                display_error("Adding file failed", &error.to_string());
            }
        });        
    }
}